@@ -86,20 +86,30 @@ sub output_wrap_vfunc_h($$$$$)
 {
   my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc) = @_;
 
-  my $str = sprintf("_VFUNC_H(%s,%s,\`%s\')dnl\n",
-    $$objCppfunc{name},
-    $$objCppfunc{rettype},
-    $objCppfunc->args_types_and_names()
-   );
-  $self->append($str);
+#TODO: We can probably remove _VFUNC_H from the .m4 file
+#  my $str = sprintf("_VFUNC_H(%s,%s,\`%s\',%s)dnl\n",
+#    $$objCppfunc{name},
+#    $$objCppfunc{rettype},
+#    $objCppfunc->args_types_and_names(),
+#    $objCppfunc->get_is_const()
+#   );
+#  $self->append($str);
 
-  #The default callback, which will call *_impl, which will then call the base default callback.
+  my $cppVfuncDecl = "virtual " . $$objCppfunc{rettype} . " " . $$objCppfunc{name} . "(" . $objCppfunc->args_types_and_names() . ")";
+  if($objCppfunc->get_is_const())
+  {
+    $cppVfuncDecl .= " const";
+  }
+
+  $self->append("  $cppVfuncDecl;");
+
+  #The default callback, which will call *_vfunc, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
 
   my $str = sprintf("_VFUNC_PH(%s,%s,\`%s\')dnl\n",
     $$objCDefsFunc{name},
     $$objCDefsFunc{rettype},
-    $objCDefsFunc->args_types_and_names()
+    $objCDefsFunc->args_types_and_names(),
    );
   $self->append($str);
 }
@@ -125,6 +135,7 @@ sub output_wrap_vfunc_cc($$$$$$)
     $$objDefsSignal{rettype},
     $objCppfunc->args_types_and_names(),
     convert_args_cpp_to_c($objCppfunc, $objDefsSignal, 0, $line_num), #$objCppfunc->args_names_only(),
+    $objCppfunc->get_is_const(),
     $refreturn);
 
   $self->append($str);
@@ -176,9 +187,9 @@ sub output_wrap_default_signal_handler_h($$$$$$)
 }
 
 # _SIGNAL_CC(signame, gtkname, rettype, crettype,`<cppargs>',`<cargs>')
-sub output_wrap_default_signal_handler_cc($$$$$)
+sub output_wrap_default_signal_handler_cc($$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $bImplement) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $bImplement, $bCustomCCallback) = @_;
   my $cname = $$objDefsSignal{name};
   # $cname = $1 if ($args[3] =~ /"(.*)"/); #TODO: What's this about?
 
@@ -208,16 +219,19 @@ sub output_wrap_default_signal_handler_cc($$$$$)
 #    $arglist_names .= $arglist_names_extra;
 #  }
 
-  my $str = sprintf("_SIGNAL_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s)dnl\n",
-    $$objCppfunc{name},
-    $cname,
-    $$objCppfunc{rettype},
-    $$objDefsSignal{rettype},
-    $objDefsSignal->args_types_and_names(),
-    $objDefsSignal->args_names_only(),
-    convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
-    ${$objDefsSignal->get_param_names()}[0]);
-  $self->append($str);
+  if($bCustomCCallback ne 1)
+  {
+    my $str = sprintf("_SIGNAL_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s)dnl\n",
+      $$objCppfunc{name},
+      $cname,
+      $$objCppfunc{rettype},
+      $$objDefsSignal{rettype},
+      $objDefsSignal->args_types_and_names(),
+      $objDefsSignal->args_names_only(),
+      convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
+      ${$objDefsSignal->get_param_names()}[0]);
+    $self->append($str);
+  }
 }
 
 ### Convert _WRAP to a method
@@ -327,27 +341,11 @@ sub output_wrap_create($$$)
   $self->append($str)
 }
 
-# _SIGNAL_IMPL(return_type,func_name,`<args>',cname)
-# sub output_wrap_sig_impl($$$$)
-# {
-#  my ($self, $filename, $line_num, $objCppfunc) = @_;
-#
-#   my $str;
-#   $str = sprintf("_SIGNAL_IMPL(%s,%s,\`%s\')dnl\n",
-#     $$objCppfunc{rettype},
-#     $$objCppfunc{name},
-#     $objCppfunc->args_types_only()
-#    );
-#
-#   $self->append($str);
-# }
-
-
-# void output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppfunc, $signal_name)
+# void output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback)
 # custom_signalproxy_name is "" when no type conversion is required - a normal templates SignalProxy will be used instead.
-sub output_wrap_sig_decl($$$$$$)
+sub output_wrap_sig_decl($$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name) = @_;
+  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback) = @_;
 
 # _SIGNAL_PROXY(c_signal_name, c_return_type, `<c_arg_types_and_names>',
 #               cpp_signal_name, cpp_return_type, `<cpp_arg_types>',`<c_args_to_cpp>',
@@ -361,6 +359,7 @@ sub output_wrap_sig_decl($$$$$$)
     $$objCppfunc{rettype},
     $objCppfunc->args_types_only(),
     convert_args_c_to_cpp($objCSignal, $objCppfunc, $line_num),
+    $bCustomCCallback, #When this is true, it will not write the *_callback implementation for you.
     $objCppfunc->get_refdoc_comment()
   );
 
@@ -458,17 +457,18 @@ sub output_wrap_property($$$$$$)
   {
     $self->output_wrap_failed($name, "property defs lookup failed.");
   }
-  elsif($objProperty->get_construct_only() eq 1)
-  {
-    $self->output_wrap_failed($name, "Attempt to wrap a construct-only property.");
-  }
   else
   {
     # We use a suffix to specify a particular Glib::PropertyProxy* class.
     my $proxy_suffix = "";
 
     # Read/Write:
-    if($objProperty->get_readable() ne 1)
+    if($objProperty->get_construct_only() eq 1)
+    {
+      # construct-only functions can be read, but not written.
+      $proxy_suffix = "_ReadOnly";
+    }
+    elsif($objProperty->get_readable() ne 1)
     {
       $proxy_suffix = "_WriteOnly";
     }
@@ -476,16 +476,29 @@ sub output_wrap_property($$$$$$)
     {
        $proxy_suffix = "_ReadOnly";
     }
-
+    
     $name =~ s/-/_/g;
 
     my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s)dnl\n",
       $name,
       $cpp_type,
       $proxy_suffix
-     );
-
+    );
     $self->append($str);
+    $self->append("\n");
+
+    # If the property is not already read-only, then add a second const accessor for a read-only propertyproxy:
+    if($proxy_suffix ne "_ReadOnly")
+    {
+      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s)dnl\n",
+        $name,
+        $cpp_type,
+        "_ReadOnly"
+      );
+      $self->append($str);
+    }
+
+    
   }
 }
 
